@@ -22,7 +22,7 @@
 /**
  * Encapsulates all audio capture operation configuration options.
  */
-var CaptureAudioOptions = function () {
+const CaptureAudioOptions = function () {
     // Upper limit of sound clips user can record. Value must be equal or greater than 1.
     this.limit = 1;
     // Maximum duration of a single sound clip in seconds.
