@@ -22,7 +22,7 @@
 /**
  * Encapsulates all video capture operation configuration options.
  */
-var CaptureVideoOptions = function () {
+const CaptureVideoOptions = function () {
     // Upper limit of videos user can record. Value must be equal or greater than 1.
     this.limit = 1;
     // Maximum duration of a single video clip in seconds.
